# .gitignore

# Node.js
node_modules/
npm-debug.log
yarn-error.log

# Logs
logs/
*.log
*.log.*

# Dependency directories
jspm_packages/
bower_components/

# Build directories
dist/
build/

# IDE files
.vscode/
.idea/
*.suo
*.ntvs*
*.njsproj
*.sln

# OS files
.DS_Store
Thumbs.db

# Angular specific
.angular/
.angular/cache/
.angular-build/
.angular-cli.json
.angular.json

# Compiled files
*.out
*.exe
*.dll
*.so
*.dylib

# Coverage directory used by tools like istanbul
coverage/

# Environment variables
.env
.env.local
.env.*.local

# Miscellaneous
*.swp
*.swo
*.bak
*.tmp
*.temp
*.log
*.log.*

# Temporary files
tmp/
temp/