[tool.poetry]
name = "{{main_project_name|replace("_","-")}}"
description = "{{main_project_name}} Kafka consumer library"
authors = ["Your Name <your.email@example.com>"]
version = "{{ "0.1.0" if not package_version else package_version }}"  # Placeholder version, dynamic versioning can be handled with plugins
packages = [{include = "{{main_project_name|lower}}", from = "src"}]

[tool.poetry.dependencies]
python = "<4.0,>=3.10"
{{data_project_name|replace("_","-")}} = {path = "../{{data_project_name}}", develop = true}
confluent-kafka = ">=2.4.0"
cloudevents = ">=1.10.1"

[tool.poetry.dev-dependencies]
pytest = ">=8.2.2"
flake8 = ">=7.1.0"
pylint = ">=3.2.3"
mypy = ">=1.10.0"
testcontainers = ">=4.5.1"
pytest-asyncio = ">=0.23.7"

[tool.pytest.ini_options]
asyncio_mode = "strict"

[build-system]
requires = ["poetry-core>=1.0.0"]
build-backend = "poetry.core.masonry.api"
