<Project Sdk="Microsoft.NET.Sdk">

  <PropertyGroup>
    <TargetFramework>net9.0</TargetFramework>
    <IsPackable>false</IsPackable>
    <Nullable>enable</Nullable>
  </PropertyGroup>

  <ItemGroup>
    {{- dependency( 'cs','net90', 'xunit') }}
    <PackageReference Include="xunit.runner.visualstudio" Version="2.8.1">
    <IncludeAssets>runtime; build; native; contentfiles; analyzers; buildtransitive</IncludeAssets>
    <PrivateAssets>all</PrivateAssets>
    </PackageReference>
    {{- dependency( 'cs','net90', 'Microsoft.NET.Test.Sdk') }}
    {{- dependency( 'cs','net90', 'Testcontainers') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents.SystemTextJson') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents.Protobuf') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents.Avro') }}
    {{- dependency( 'cs','net90', 'MQTTnet') }}
    {{- dependency( 'cs','net90', 'Azure.Storage.Blobs') }}
    {{- dependency( 'cs','net90', 'Microsoft.Extensions.Logging') }}
    {{- dependency( 'cs','net90', 'Microsoft.Extensions.Logging.Console') }}
    {{- dependency( 'cs','net90', 'Microsoft.Extensions.Logging.Debug') }}
    {{- dependency( 'cs','net90', 'System.Memory.Data') }}
  </ItemGroup>

  <ItemGroup>
    <ProjectReference Include="..\src\{{ main_project_name | pascal }}.csproj" />
    <ProjectReference Include="..\..\{{ data_project_name }}\test\{{data_project_name | pascal}}.Test.csproj" />
  </ItemGroup>

</Project>
