{
  "name": "{{project_name|dotdash|lower}}-dashboard-backend",
  "version": "1.0.0",
  "description": "Backend server for dashboard",
  "type": "module",
  "main": "dist/index.js",
  "scripts": {
    "build": "tsc && cpx \"public/**/*\" dist/public",
    "start": "node dist/index.js",
    "dev": "ts-node src/index.ts",
    "debug": "nodemon --inspect --watch 'src/**/*.ts' --exec 'ts-node' src/index.ts"
  },
  "keywords": [],
  "author": "",
  "license": "ISC",
  "dependencies": {
    "async-sema": "3.1.1",
    "azure-kusto-data": "^6.0.2",
    "cloudevents": "^8.0.2",
    "dotenv": "^16.4.5",
    "express": "^4.21.0",
    "kafkajs": "^2.2.4",
    "pako": "^2.1.0",
    "socket.io": "^4.8.0",
    "winston": "^3.14.2"
  },
  "devDependencies": {
    "@types/express": "^4.17.21",
    "@types/node": "^22.7.5",
    "cpx": "^1.5.0",
    "nodemon": "^3.0.1",
    "ts-node": "^10.9.2",
    "typescript": "^5.6.3"
  }
}
