{
    "ScriptType": "JobConfig",
    "DataLocale": "en-US",
    "OutputErrorPolicy": "Retry",
    "EventsLateArrivalMaxDelayInSeconds": 5,
    "EventsOutOfOrderMaxDelayInSeconds": 0,
    "EventsOutOfOrderPolicy": "Adjust",
    "StreamingUnits": 3,
    "CompatibilityLevel": "1.2",
    "UseSystemAssignedIdentity": false,
    "GlobalStorage": {
        "AccountName": null,
        "AccountKey": null,
        "AuthenticationMode": "ConnectionString"
    },
    "ContentStoragePolicy": "SystemAccount",
    "CustomCodeStorage": {
        "AccountName": null,
        "AccountKey": null,
        "ContainerName": null,
        "Path": "UserCustomCode.zip"
    },
    "VirtualNetworkConfiguration": {
        "VirtualNetworkName": null,
        "SubnetResourceId": null
    },
    "DataSourceCredentialDomain": null,
    "Tags": {}
}